/*
 *  Copyright (C) 1997-2001 B. Cameron Lesiuk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ANSITERM_H_
#define _ANSITERM_H_

#include "termbuf.h"

#define AT_WIND_CLASS_NAME "AnsiTerm:BCL 98/05/02"

/* Window extra data offsets */
#define ATWND_ATTERMSEG 0  /* pointer to the window's ATTERM structure */
#define ATWND_ATTERMOFF 4  /* pointer to the window's ATTERM structure */

/* Emulation Modes */
#define AT_EMU_NONE   0
#define AT_EMU_VT100  1

/* Constants */
#define TBSTR_TITLE_LEN     200     /* max # chars in title of window */
#define TBSTR_ENVSECTION    200     /* max size of environment string */
#define TBMACRO_MAXLEN      200
#define AT_ESC_PARAM_MAX    50
#define AT_STATEVAL_DEFAULT -1

/* Effect bits */
#define AT_FONTEFFECT_BOLD      (1<<0)
#define AT_FONTEFFECT_UNDERLINE (1<<1)
#define AT_FONTEFFECT_BLINK     (1<<2)
#define AT_FONTEFFECT_REVERSE   (1<<3)

/* aNotify messages */
#define TBNOTIFY_MENUCLOSE   0x0001 /* user close from the menu or system menu */
#define TBNOTIFY_USERINPUT   0x0002 /* user input is available for transmission */
#define TBNOTIFY_OPTIONS     0x0003 /* user options have changed                */
#define TBNOTIFY_OPTIONSHELP 0x0004 /* options Help requested                   */
#define TBNOTIFY_FONT        0x0005 /* user has selected a new display font     */
#define TBNOTIFY_FONTHELP    0x0006 /* font picker help requested               */
#define TBNOTIFY_MACROS      0x0007 /* user changed function key macros         */
#define TBNOTIFY_MACROSHELP  0x0008 /* function key macros help requested       */
#define ATNOTIFY_WINDOWSIZE  0x0009 /* Terminal window size has changed         */
#define ATNOTIFY_DESTROY     0x000A /* Terminal window has been closed          */


typedef struct ATINPUTLINEtag {
	HGLOBAL aMemory;        /* global handle for this memory object */
	struct ATINPUTLINEtag *aNext,*aPrev;
	unsigned char aData[5]; /* we alloc a wack of data and typecast it to this type.
                           * we then just use aData[] to index. Note we expect to overrun the aData size of 5!!! */
} ATINPUTLINE;

typedef struct ATTERMtag {
	HINSTANCE     aInst;               /* instance to which this term belongs */
	HGLOBAL       aMemory;             /* our memory handle */
	HGLOBAL       aOutBufMemory;       /* output buffer memory */
	unsigned char *aOutBuf;            /* output buffer */
	unsigned long aOutBufSize;         /* output buffer size */
	unsigned char *aOutBufSockTail;    /* outgoing socket position (tail) */
//    char          *aOutBufLETail;      /* local echo position */
	unsigned char *aOutBufHead;        /* head position */
	HWND          aWnd;                /* our window */
	TBTERMBUF     *aTermBuf;           /* our terminal buffer */
	long    aShowX, aShowY;       /* the screen coordinate of the top-left shown cell */
	long    aSizeX, aSizeY;       /* size of visible screen - updated by atAdjustTerm */
	long    aScroll;              /* # lines to scroll up (neg for down) next WM_PAINT */
	HFONT   aFont;                /* Changed by atChangeFont */
	HFONT   aFontUnderline;       /* Changed by atChangeFont */
	LOGFONT aLogFont;             /* Changed by atChangeFont */
	BOOL    aLogFontValid;        /* TRUE if aLogFont is valid */
	long    aFontX, aFontY;       /* Font cell width,height. Updated by atChangeFont */
	int     *aState;              /* Terminal state for the parsing state machine */
	long    aStateVal[AT_ESC_PARAM_MAX];   /* parsed state variable array (if req'd) */
	long    aStateValPos;
	char    aEnvSection[TBSTR_ENVSECTION];/* environment (window position) section */
	char    aTitle[TBSTR_TITLE_LEN]; /* Window title */
	void    (*aNotify)(struct ATTERMtag *atTerm, int msg);    /* function called to notify of out-bound data is available */
	long    aLE_BSCount;     /* used to allow only x backspaces during local echo */

	/* Mouse hilite settings */
	BOOL aMouseHiliteCapture; /* 0 normally, 1 if we've captured the mouse input */
	int  aCaptureLastX, aCaptureLastY; /* the last location of the mouse, relative to client screen */

	unsigned long aEmulation;
	BOOL          aLocalEcho;     /* TRUE for local echo */
	BOOL          aBell;          /* audible bell */
	unsigned char aBackSpaceChar; /* backspace key character */
	unsigned char aDeleteChar;    /* delete character */

	/* Other terminal settings */
	unsigned char aFnKeyMacro[48][TBMACRO_MAXLEN];  /* plain, shift, control, shift-control */

	/*    Modes                         TRUE      FALSE    */
	BOOL  aVLocalEcho;               /* on        off      */
	BOOL  aLED[4];                   /* on        off      */
	BOOL  aLFwithNLMode;             /* LF+NL     LF       */
	BOOL  aCursorKeyAppMode;         /* app       cursor   */
	BOOL  aANSIMode;                 /* ANSI      VT52     */
	BOOL  aColumnMode;               /* 132       80       */
	BOOL  aScrollingMode;            /* Smooth    Jump     */
	BOOL  aScreenMode;               /* Reverse   Normal   */
	BOOL  aOriginMode;               /* Relative  Absolute */
	BOOL  aWraparoundMode;           /* on        off      */
	BOOL  aAutoRepeatMode;           /* on        off      */

	int   aSCSType; /* temporary holding place for state machine -- NOT a terminal setting!!! */

	/* ECMA-48 Codes and Modes */
	int   aSIMD;               /* Select Implicit Movement Direction */
	int   aSLH;                /* Set Line Home - home position (starts at 1) */
	int   aSLL;                /* Set Line Limit */

} ATTERM;



BOOL   vt100InitClass(HINSTANCE hInst);
ATTERM *vt100CreateWnd(HINSTANCE hInst, HWND parent,long bufX,long bufY, long scrX, long scrY, char *envSection);
void   vt100DestroyWnd(ATTERM *atTerm);
void   vt100ChangeFont(ATTERM *atTerm, HFONT font, LOGFONT *logfont);
void   vt100SaveSettings(ATTERM *atTerm, int pSettings, char *pSection);
void   vt100LoadSettings(ATTERM *atTerm, int pSettings, char *pSection);

int    vt100Send(ATTERM *atTerm, unsigned char *pBuf, int pBufLen);
long   vt100ReadInput(ATTERM *atTerm, unsigned char *pBuf, unsigned long pBufLen);


void atAdjustTerm(ATTERM *atTerm,long p_Scrollback);
void atUpdateTermTitle(ATTERM *atTerm,char *newTitle);
void atAdjustScrollbar(ATTERM *atTerm);

void atShowTerm(ATTERM *atTerm);
void atHideTerm(ATTERM *atTerm);
BOOL atIsTermVisible(ATTERM *atTerm);

void atClearTerm(ATTERM *atTerm);

void atNotifyFn(unsigned long p_ID, unsigned int p_Operation, long p_P0,long p_P1, long p_P2, long p_P3);
LRESULT CALLBACK atAnsiTermProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
void atUpdateCaret(ATTERM *atTerm);
void atScrollTermTo(ATTERM *atTerm, BOOL p_Rel, long p_Val);

long atUnReadTermInput(ATTERM *atTerm, unsigned char *pBuf, unsigned long pBufLen);
long atWriteTermInput(ATTERM *atTerm, unsigned char *pBuf, unsigned long pBufLen);
void atSendTermInput(ATTERM *atTerm, unsigned char *pBuf, unsigned long pBufLen);

void atResetTerm(ATTERM *atTerm);

#endif /* _ANSITERM_H_ */


