/*
 *  Copyright (C) 1997-2001 B. Cameron Lesiuk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Version: 2.1.0.0 - Aug 25 2003 11:11:11
 */

#include <windows.h>
#include <stdio.h>

/* Constants used by this module */
#define REG_ROOT      "k+b"
#define REG_PRODUCT   "WS68"



/*
 * Opens the root registry key for the current application.
 * It goes along the path HKEY_CURRENT_USER->"Software"->EN_ROOT->EN_PRODUCT
 * and returns the key for the EN_PRODUCT level key. All intermediate keys are
 * automatically closed. The calling function is responsible for closing the
 * returned key
 */
HKEY enGetRootRegistryKey(void)
{
	long  reg;
	HKEY  key1, key2;
	DWORD disp;

	reg = RegCreateKeyEx(HKEY_CURRENT_USER, "Software", 0, NULL, 0, KEY_READ, NULL, &key1, &disp);
	if (reg == ERROR_SUCCESS) {
		reg = RegCreateKeyEx(key1, REG_ROOT, 0, NULL,
				REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key2, &disp);
		RegCloseKey(key1);
		key1 = key2;
		if (reg == ERROR_SUCCESS) {
			reg = RegCreateKeyEx(key1, REG_PRODUCT, 0, NULL,
					REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key2,&disp);
			RegCloseKey(key1);
			key1 = key2;
			if (reg == ERROR_SUCCESS) {
				return key1;
			}
		}
	}

	return 0;
}


/*
 * This proc wraps the old WritePrivateProfileString function and
 * does registry calls for O/S's greater than Win3.11
 */
BOOL enWritePrivateProfileString(char *pSection,char *pEntry, char *pString)
{
	long  reg;
	HKEY  root, section;
	DWORD disp;
	BOOL  rc = FALSE;

	root = enGetRootRegistryKey();
	if (root) {
		reg = RegCreateKeyEx(root, pSection, 0, NULL,
					REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS, NULL, &section, &disp);
		if (reg == ERROR_SUCCESS) {
			reg = RegSetValueEx(section, pEntry, 0, REG_SZ, (BYTE*)pString, strlen(pString)+1);
			if (reg == ERROR_SUCCESS) {
				rc = TRUE;
			}
			RegCloseKey(section);
		}
		RegCloseKey(root);
	}

	return rc;
}


UINT enGetPrivateProfileInt(char *pSection, char *pEntry, int pDefault)
{
	long  reg;
	HKEY  root, section;
	DWORD disp, type, len;
	int   rc = pDefault;
	char  buf[100];

	root = enGetRootRegistryKey();
	if (root) {
		reg = RegCreateKeyEx(root, pSection, 0, NULL,
				REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &section, &disp);
		if (reg == ERROR_SUCCESS) {
			len = 100;
			reg = RegQueryValueEx(section, pEntry, NULL, &type, (BYTE*)buf, &len);
			if ((reg == ERROR_SUCCESS) && (type == REG_SZ)) {
				sscanf(buf, "%u", &rc);
			}
			RegCloseKey(section);
		}
		RegCloseKey(root);
	}
	return rc;
}


int enGetPrivateProfileString(char *pSection, char *pEntry, char *pDefault, char *pBuf, int pBufSize)
{
    long  reg;
    HKEY  root, section;
    DWORD disp, type, len;
    int   rc;

    strcpy(pBuf, pDefault);
    rc = strlen(pBuf);
    root = enGetRootRegistryKey();
    if (root) {
      reg = RegCreateKeyEx(root,pSection,0,NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&section,&disp);
      if (reg == ERROR_SUCCESS) {
        len = pBufSize;
        reg = RegQueryValueEx(section, pEntry, NULL, &type, (BYTE*)pBuf, &len);
        if ((reg == ERROR_SUCCESS) && (type == REG_SZ)) {
          rc = len;
        } else {
          strcpy(pBuf, pDefault);
          rc = strlen(pBuf);
        }
        RegCloseKey(section);
      }
      RegCloseKey(root);
    }

    return rc;
}



WINDOWPLACEMENT *enRestoreWindowPlacement(char *section,int p_defLeft,
  int p_defTop,int p_defRight,int p_defBottom)
{
    static WINDOWPLACEMENT wp;

    if (!section)  return NULL;

    wp.length = sizeof(WINDOWPLACEMENT);
    wp.flags  = (UINT)enGetPrivateProfileInt(section,"flags", 0);
    wp.showCmd= (UINT)enGetPrivateProfileInt(section,"showCmd", SW_SHOWNA);
    wp.ptMinPosition.x = (int)enGetPrivateProfileInt(section,"ptMinX", 100);
    wp.ptMinPosition.y = (int)enGetPrivateProfileInt(section,"ptMinY", 100);
    wp.ptMaxPosition.y = (int)enGetPrivateProfileInt(section,"ptMaxY",   0);
    wp.ptMaxPosition.y = (int)enGetPrivateProfileInt(section,"ptMaxY",   0);
    wp.rcNormalPosition.left  = (int)enGetPrivateProfileInt(section,"rcNormalLeft", p_defLeft);
    wp.rcNormalPosition.top   = (int)enGetPrivateProfileInt(section,"rcNormalTop", p_defTop);
    wp.rcNormalPosition.right = (int)enGetPrivateProfileInt(section,"rcNormalRight", p_defRight);
    wp.rcNormalPosition.bottom= (int)enGetPrivateProfileInt(section,"rcNormalBottom", p_defBottom);

    return &wp;
}


void enSaveWindowPlacement(char *section, HWND hWnd) 
{
    static WINDOWPLACEMENT wp;
	char buf[30];

	if ((!section) || (!hWnd))
		return;

	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(hWnd, &wp);

	sprintf(buf, "%i", wp.length);
	enWritePrivateProfileString(section, "length", buf);
	sprintf(buf, "%i", wp.flags);
	enWritePrivateProfileString(section, "flags", buf);
	sprintf(buf, "%i", wp.showCmd);
	enWritePrivateProfileString(section, "showCmd", buf);
	sprintf(buf, "%i", wp.ptMinPosition.x);
	enWritePrivateProfileString(section, "ptMinX", buf);
	sprintf(buf, "%i", wp.ptMinPosition.y);
	enWritePrivateProfileString(section, "ptMinY", buf);
	sprintf(buf, "%i", wp.ptMaxPosition.x);
	enWritePrivateProfileString(section, "ptMaxX", buf);
	sprintf(buf, "%i", wp.ptMaxPosition.y);
	enWritePrivateProfileString(section, "ptMaxY", buf);
	sprintf(buf, "%i", wp.rcNormalPosition.left);
	enWritePrivateProfileString(section, "rcNormalLeft", buf);
	sprintf(buf, "%i", wp.rcNormalPosition.top);
	enWritePrivateProfileString(section, "rcNormalTop", buf);
	sprintf(buf, "%i", wp.rcNormalPosition.right);
	enWritePrivateProfileString(section, "rcNormalRight", buf);
	sprintf(buf, "%i", wp.rcNormalPosition.bottom);
	enWritePrivateProfileString(section, "rcNormalBottom", buf);

	return;
}


void enSaveLong(char *pSection, char *pEntry, unsigned long pLong) {
	char buf[20];

	sprintf(buf, "%lu", pLong);
	enWritePrivateProfileString(pSection, pEntry, buf);

	return;
}


unsigned long enRestoreLong(char *pSection, char *pEntry, unsigned long pDefault)
{
	char buf[20], buf2[20];
	unsigned long l;

	sprintf(buf2, "%lu", pDefault);
	enGetPrivateProfileString(pSection, pEntry, buf2, buf, 20);
	sscanf(buf,"%lu",&l);

	return l;
}


void enSaveInt(char *pSection, char *pEntry, unsigned int pInt) {
	char buf[20];

	sprintf(buf, "%u", pInt);
	enWritePrivateProfileString(pSection, pEntry, buf);

	return;
}


unsigned int enRestoreInt(char *pSection, char *pEntry, unsigned int pDefault)
{
	char  buf[20], buf2[20];
	unsigned int i;

	sprintf(buf2, "%u", pDefault);
	enGetPrivateProfileString(pSection, pEntry, buf2, buf, 20);
	sscanf(buf, "%u", &i);

	return i;
}


void enSaveString(char *pSection, char *pEntry, unsigned char *pStr)
{
	enWritePrivateProfileString(pSection, pEntry, (char *)pStr);
}


void enRestoreString(char *pSection, char *pEntry, char *pDefault, unsigned char *pStr, unsigned long pStrLen)
{
	enGetPrivateProfileString(pSection, pEntry, pDefault, (char*)pStr, pStrLen);
}


void enSaveYesNo(char *pSection, char *pEntry, BOOL yesno)
{
	if (yesno)
		enWritePrivateProfileString(pSection, pEntry, "Yes");
	else
		enWritePrivateProfileString(pSection, pEntry, "No");
}


BOOL enRestoreYesNo(char *pSection, char *pEntry, BOOL pDefault)
{
	char  buf[20], *def;

	if (pDefault)
		def = "Yes";
	else
		def = "No";
	enGetPrivateProfileString(pSection, pEntry, def, buf, 20);
	switch(buf[0]) {
		case 'y':
		case 'Y':
		case '1':
			return TRUE;
		case 'n':
		case 'N':
		case '0':
		default:
			return FALSE;
	}
}


void enSaveLogFont(char *section, LOGFONT *logFont)
{
	enSaveLong(section, "lfHeight", (unsigned long)logFont->lfHeight);
	enSaveLong(section, "lfWidth",  (unsigned long)logFont->lfWidth);
	enSaveLong(section, "lfEscapement", (unsigned long)logFont->lfEscapement);
	enSaveLong(section, "lfOrientation", (unsigned long)logFont->lfOrientation);
	enSaveLong(section, "lfWeight", (unsigned long)logFont->lfWeight);
	enSaveInt(section, "lfItalic", (unsigned int)logFont->lfItalic);
	enSaveInt(section, "lfUnderline", (unsigned int)logFont->lfUnderline);
	enSaveInt(section, "lfStrikeOut", (unsigned int)logFont->lfStrikeOut);
	enSaveInt(section, "lfCharSet", (unsigned int)logFont->lfCharSet);
	enSaveInt(section, "lfOutPrecision", (unsigned int)logFont->lfOutPrecision);
	enSaveInt(section, "lfClipPrecision", (unsigned int)logFont->lfClipPrecision);
	enSaveInt(section, "lfQuality", (unsigned int)logFont->lfQuality);
	enSaveInt(section, "lfPitchAndFamily", (unsigned int)logFont->lfPitchAndFamily);
	enSaveString(section, "lfFaceName", (unsigned char*)logFont->lfFaceName);
}


void enRestoreLogFont(char *section, LOGFONT *logFont)
{
	logFont->lfHeight = (long)enRestoreLong(section, "lfHeight", 0L);
	logFont->lfWidth  = (long)enRestoreLong(section, "lfWidth", 0L);
	logFont->lfEscapement = (long)enRestoreLong(section, "lfEscapement", 0L);
	logFont->lfOrientation= (long)enRestoreLong(section, "lfOrientation", 0L);
	logFont->lfWeight = (long)enRestoreLong(section, "lfWeight", 0L);
	logFont->lfItalic = (BYTE)enRestoreInt(section,  "lfItalic", 0);
	logFont->lfUnderline = (BYTE)enRestoreInt(section, "lfUnderline", 0);
	logFont->lfStrikeOut = (BYTE)enRestoreInt(section, "lfStrikeOut", 0);
	logFont->lfCharSet   = (BYTE)enRestoreInt(section, "lfCharSet", 0);
	logFont->lfOutPrecision = (BYTE)enRestoreInt(section, "lfOutPrecision", 0);
	logFont->lfClipPrecision= (BYTE)enRestoreInt(section, "lfClipPrecision", 0);
	logFont->lfQuality = (BYTE)enRestoreInt(section, "lfQuality", 0);
	logFont->lfPitchAndFamily = (BYTE)enRestoreInt(section, "lfPitchAndFamily", 0);
	enRestoreString(section, "lfFaceName", "", (unsigned char*) logFont->lfFaceName, LF_FACESIZE);
}
